#include <linux/version.h>

#include <linux/string.h>
#include <linux/errno.h>
#include <linux/module.h>

#include "nikal.h"

#ifdef nNIKAL100_kCopyrightString
   MODULE_LICENSE(nNIKAL100_kCopyrightString);
#endif

#ifdef nNIKAL100_kAuthorString
   MODULE_AUTHOR(nNIKAL100_kAuthorString);
#endif

#ifdef nNIKAL100_kDescriptionString
   MODULE_DESCRIPTION(nNIKAL100_kDescriptionString);
#endif


#include <linux/pci.h>
#include <linux/mod_devicetable.h>

int nNIAPAL200_driverEntry(struct nNIKAL100_tDriver *driver);

static struct nNIKAL100_tDriver atomicDriver;
static void* nativeDriverRegistration;

/* native driver registration functions. These functions are auto-generated */
void *registerNativeDriver(nNIKAL100_tDriver*);
void unregisterNativeDriver(void*);

int init_module(void)
{
   atomicDriver.module = THIS_MODULE;

   if (nNIAPAL200_driverEntry(&atomicDriver))
      return -ENODEV;

   nativeDriverRegistration = registerNativeDriver(&atomicDriver);

   if (nativeDriverRegistration == NULL)
   {
      if (atomicDriver.unload)
         atomicDriver.unload(&atomicDriver);
      return -ENODEV;
   }

   return 0;
}

void cleanup_module(void)
{
   unregisterNativeDriver(nativeDriverRegistration);

   if (atomicDriver.unload)
      atomicDriver.unload(&atomicDriver);
}

static struct pci_device_id nipciidtable[] = {
   {0x1093, 0x7152, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x73A5, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7042, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7082, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x702E, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x702F, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7193, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7194, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7055, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7056, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x702C, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x702D, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7074, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7083, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7195, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7390, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7393, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7391, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7394, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7384, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7386, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7385, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7387, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7392, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x73E1, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x75B1, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7293, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x71C7, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x73ED, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x729D, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x70F6, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x705E, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x70F7, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x70F8, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7373, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7374, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7375, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7397, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7398, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7399, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x739A, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7378, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x739B, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x73F5, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7626, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7627, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7539, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x753A, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x73F6, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x73F7, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x73F8, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x73F9, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x73FA, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x73FB, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x74E8, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x751B, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x751C, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
    { },
};
MODULE_DEVICE_TABLE(pci, nipciidtable);

static struct pci_driver niPciDriver = {
   .name = "niriomtk",
   .id_table = nipciidtable,
};

void *registerNativeDriver(nNIKAL100_tDriver* ignore)
{
   return nNIKAL200_registerPCIDriver(&niPciDriver);
}
void unregisterNativeDriver(void* data)
{
   nNIKAL200_unregisterPCIDriver(data);
}


#define nNIKAL100_mExportSymbol EXPORT_SYMBOL
